@@ -18,32 +18,54 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import traceback
 from operator import itemgetter
-from PyQt5 import QtCore, QtNetwork
-from picard import config, log
-from picard.coverart import coverart
-from picard.metadata import (Metadata,
-                             register_album_metadata_processor,
-                             run_album_metadata_processors,
-                             run_track_metadata_processors)
-from picard.dataobj import DataObject
-from picard.file import File
-from picard.track import Track
-from picard.script import ScriptParser, ScriptError, enabled_tagger_scripts_texts
-from picard.ui.item import Item
-from picard.util import format_time, mbid_validate
-from picard.util.imagelist import update_metadata_images
-from picard.util.textencoding import asciipunct
+import traceback
+
+from PyQt5 import (
+    QtCore,
+    QtNetwork,
+)
+
+from picard import (
+    config,
+    log,
+)
 from picard.cluster import Cluster
 from picard.collection import add_release_to_user_collections
+from picard.const import VARIOUS_ARTISTS_ID
+from picard.coverart import coverart
+from picard.dataobj import DataObject
+from picard.file import File
 from picard.mbjson import (
+    medium_to_metadata,
     release_group_to_metadata,
     release_to_metadata,
-    medium_to_metadata,
     track_to_metadata,
 )
-from picard.const import VARIOUS_ARTISTS_ID
+from picard.metadata import (
+    Metadata,
+    register_album_metadata_processor,
+    run_album_metadata_processors,
+    run_track_metadata_processors,
+)
+from picard.script import (
+    ScriptError,
+    ScriptParser,
+    enabled_tagger_scripts_texts,
+)
+from picard.track import Track
+from picard.util import (
+    format_time,
+    mbid_validate,
+)
+from picard.util.imagelist import (
+    add_metadata_images,
+    remove_metadata_images,
+    update_metadata_images,
+)
+from picard.util.textencoding import asciipunct
+
+from picard.ui.item import Item
 
 register_album_metadata_processor(coverart)
 
@@ -151,7 +173,7 @@ class Album(DataObject, Item):
         # Run album metadata plugins
         try:
             run_album_metadata_processors(self, m, release_node)
-        except:
+        except BaseException:
             self.error_append(traceback.format_exc())
 
         self._release_node = release_node
@@ -250,7 +272,7 @@ class Album(DataObject, Item):
                         track = self._finalize_loading_track(track_node, mm, artists, va, absolutetracknumber, discpregap)
                         track.metadata['~datatrack'] = "1"
 
-            totalalbumtracks = string_(absolutetracknumber)
+            totalalbumtracks = str(absolutetracknumber)
 
             for track in self._new_tracks:
                 track.metadata["~totalalbumtracks"] = totalalbumtracks
@@ -319,13 +341,15 @@ class Album(DataObject, Item):
         artists.add(tm["artist"])
         if va:
             tm["compilation"] = "1"
+        else:
+            tm.delete("compilation")
         if discpregap:
             tm["~discpregap"] = "1"
 
         # Run track metadata plugins
         try:
             run_track_metadata_processors(self, tm, self._release_node, track_node)
-        except:
+        except BaseException:
             self.error_append(traceback.format_exc())
 
         return track
@@ -342,9 +366,9 @@ class Album(DataObject, Item):
         self.status = _("[loading album information]")
         if self.release_group:
             self.release_group.loaded = False
-            self.release_group.folksonomy_tags.clear()
+            self.release_group.genres.clear()
         self.metadata.clear()
-        self.folksonomy_tags.clear()
+        self.genres.clear()
         self.update()
         self._new_metadata = Metadata()
         self._new_tracks = []
@@ -353,16 +377,14 @@ class Album(DataObject, Item):
         require_authentication = False
         inc = ['release-groups', 'media', 'recordings', 'artist-credits',
                'artists', 'aliases', 'labels', 'isrcs', 'collections']
+        if self.tagger.webservice.oauth_manager.is_authorized():
+            require_authentication = True
+            inc += ['user-collections']
         if config.setting['release_ars'] or config.setting['track_ars']:
             inc += ['artist-rels', 'release-rels', 'url-rels', 'recording-rels', 'work-rels']
             if config.setting['track_ars']:
                 inc += ['recording-level-rels', 'work-level-rels']
-        if config.setting['folksonomy_tags']:
-            if config.setting['only_my_tags']:
-                require_authentication = True
-                inc += ['user-tags']
-            else:
-                inc += ['tags']
+        require_authentication = self.set_genre_inc_params(inc) or require_authentication
         if config.setting['enable_ratings']:
             require_authentication = True
             inc += ['user-ratings']
@@ -388,14 +410,14 @@ class Album(DataObject, Item):
     def _add_file(self, track, file):
         self._files += 1
         self.update(update_tracks=False)
+        add_metadata_images(self, [file])
         file.metadata_images_changed.connect(self.update_metadata_images)
-        self.update_metadata_images()
 
     def _remove_file(self, track, file):
         self._files -= 1
         self.update(update_tracks=False)
         file.metadata_images_changed.disconnect(self.update_metadata_images)
-        self.update_metadata_images()
+        remove_metadata_images(self, [file])
 
     def match_files(self, files, use_recordingid=True):
         """Match files to tracks on this album, based on metadata similarity or recordingid."""
@@ -434,7 +456,7 @@ class Album(DataObject, Item):
         tracknumber = file.metadata['tracknumber']
         discnumber = file.metadata['discnumber']
         for track in self.tracks:
-            tm = track.metadata
+            tm = track.orig_metadata
             if recordingid == tm['musicbrainz_recordingid']:
                 if tracknumber == tm['tracknumber']:
                     if discnumber == tm['discnumber']:
@@ -540,10 +562,10 @@ class Album(DataObject, Item):
                 number_of_images = len(metadata.images)
                 if getattr(metadata, 'has_common_images', True):
                     text += ngettext("; %i image", "; %i images",
-                                      number_of_images) % number_of_images
+                                     number_of_images) % number_of_images
                 else:
                     text += ngettext("; %i image not in all tracks", "; %i different images among tracks",
-                                      number_of_images) % number_of_images
+                                     number_of_images) % number_of_images
                 return text + ')'
             else:
                 return title
@@ -602,9 +624,11 @@ class NatAlbum(Album):
 
     def update(self, update_tracks=True):
         self.enable_update_metadata_images(False)
+        old_album_title = self.metadata["album"]
         self.metadata["album"] = config.setting["nat_name"]
         for track in self.tracks:
-            track.metadata["album"] = self.metadata["album"]
+            if old_album_title == track.metadata["album"]:
+                track.metadata["album"] = self.metadata["album"]
             for file in track.linked_files:
                 track.update_file_metadata(file)
         self.enable_update_metadata_images(True)
